package electrical_system is
	nature electrical is real across real through;
	terminal ground : electrical;
end electrical_system;

use electrical_system.all;

package radiation_system is
	nature radiation is real across real through;
	terminal rad_ground : radiation;
end radiation_system;

use radiation_system.all;


entity radmos is
	generic ( n : real := 1.0;	-- belta
                  alpha: real :=1.0;    -- empirical factor
                  vthadd: real :=0.2;   -- "additional" Vt for unradiated
		  vthres: real :=0.5);	-- threshold voltage
	port ( terminal drn, gat, src : electrical;
               terminal radin, ref : radiation );
end radmos;

architecture arch1 of radmos is
	quantity vgs across gat to src;
	quantity vds across drn to src;
	quantity ids_d through drn;
	quantity ids_s through src;
        quantity radtotal across radflux through radin to ref;
begin
  MOSequations: procedural is
	variable idstmp : real;
  begin -- procedural statements

	IF vds > 0 THEN			-- forward
	   IF vgs <= (vthres+vthadd*exp(radtotal*alpha)) THEN	-- cut off
		idstmp := 0.0;
	   ELSE
		IF vgs-(vthres+vthadd*exp(radtotal*alpha))  <= vds THEN	
-- saturation
		   idstmp := n * (((vgs - 
(vthres+vthadd*exp(radtotal*alpha)) )**2.0) / 2.0);
	   	ELSE					-- triode
		   idstmp1 := n * ((vgs - 
(vthres+vthadd*exp(radtotal*alpha)) )*vds - ((vds**2.0)/2.0));
		   idstmp := idstmp1;
		END IF;
	   END IF;
	ELSE idstmp := 0.0;
	END IF;
	ids1 := idstmp;
	ids_d := ids1;
	ids_s := -ids_d;
        radflux := radtotal'dot; --integrate total radiation dose
  end procedural;
end architecture arch1;
